@@ -55,8 +55,11 @@ KateFileTree::KateFileTree(QWidget *parent): QTreeView(parent)
     setAcceptDrops(false);
     setIndentation(12);
     setAllColumnsShowFocus(true);
+    setFocusPolicy(Qt::NoFocus);
 
+    // handle activated (e.g. for pressing enter) + clicked (to avoid to need to do double-click e.g. on Windows)
     connect(this, SIGNAL(activated(QModelIndex)), this, SLOT(mouseClicked(QModelIndex)));
+    connect(this, SIGNAL(clicked(QModelIndex)), this, SLOT(mouseClicked(QModelIndex)));
 
     m_filelistReloadDocument = new QAction(QIcon::fromTheme(QLatin1String("view-refresh")), i18nc("@action:inmenu", "Reloa&d"), this);
     connect(m_filelistReloadDocument, SIGNAL(triggered(bool)), SLOT(slotDocumentReload()));
@@ -66,7 +69,15 @@ KateFileTree::KateFileTree(QWidget *parent): QTreeView(parent)
     connect(m_filelistCloseDocument, SIGNAL(triggered()), this, SLOT(slotDocumentClose()));
     m_filelistCloseDocument->setWhatsThis(i18n("Close the current document."));
 
-    m_filelistCloseOtherDocument = new QAction(QIcon::fromTheme(QLatin1String("window-close")), i18nc("@action:inmenu", "Close Other"), this);
+    m_filelistExpandRecursive = new QAction(QIcon::fromTheme(QLatin1String("view-list-tree")), i18nc("@action:inmenu", "Expand recursively"), this);
+    connect(m_filelistExpandRecursive, SIGNAL(triggered()), this, SLOT(slotExpandRecursive()));
+    m_filelistExpandRecursive->setWhatsThis(i18n("Expand the file list sub tree recursively."));
+
+    m_filelistCollapseRecursive = new QAction(QIcon::fromTheme(QLatin1String("view-list-tree")), i18nc("@action:inmenu", "Collapse recursively"), this);
+    connect(m_filelistCollapseRecursive, SIGNAL(triggered()), this, SLOT(slotCollapseRecursive()));
+    m_filelistCollapseRecursive->setWhatsThis(i18n("Collapse the file list sub tree recursively."));
+
+    m_filelistCloseOtherDocument = new QAction(QIcon::fromTheme(QLatin1String("document-close")), i18nc("@action:inmenu", "Close Other"), this);
     connect(m_filelistCloseOtherDocument, SIGNAL(triggered()), this, SLOT(slotDocumentCloseOther()));
     m_filelistCloseOtherDocument->setWhatsThis(i18n("Close other documents in this folder."));
 
@@ -127,7 +138,7 @@ KateFileTree::~KateFileTree()
 
 void KateFileTree::setModel(QAbstractItemModel *model)
 {
-    Q_ASSERT(qobject_cast<KateFileTreeProxyModel *>(model)); // we dont' realy work with anything else
+    Q_ASSERT(qobject_cast<KateFileTreeProxyModel *>(model)); // we don't really work with anything else
     QTreeView::setModel(model);
 }
 
@@ -222,6 +233,8 @@ void KateFileTree::contextMenuEvent(QContextMenuEvent *event)
     QMenu menu;
     menu.addAction(m_filelistReloadDocument);
     menu.addAction(m_filelistCloseDocument);
+    menu.addAction(m_filelistExpandRecursive);
+    menu.addAction(m_filelistCollapseRecursive);
 
     if (isFile) {
         menu.addAction(m_filelistCloseOtherDocument);
@@ -334,6 +347,50 @@ void KateFileTree::slotDocumentClose()
     KTextEditor::Editor::instance()->application()->closeDocuments(closingDocuments);
 }
 
+void KateFileTree::slotExpandRecursive()
+{
+    if (! m_indexContextMenu.isValid()) {
+        return;
+    }
+
+    // Work list for DFS walk over sub tree
+    QList<QPersistentModelIndex> worklist = { m_indexContextMenu };
+
+    while (! worklist.isEmpty()) {
+        QPersistentModelIndex index = worklist.takeLast();
+
+        // Expand current item
+        expand(index);
+
+        // Append all children of current item
+        for (int i=0 ; i < model()->rowCount(index) ; ++i) {
+            worklist.append(index.child(i,0));
+        }
+    }
+}
+
+void KateFileTree::slotCollapseRecursive()
+{
+    if (! m_indexContextMenu.isValid()) {
+        return;
+    }
+
+    // Work list for DFS walk over sub tree
+    QList<QPersistentModelIndex> worklist = { m_indexContextMenu };
+
+    while (! worklist.isEmpty()) {
+        QPersistentModelIndex index = worklist.takeLast();
+
+        // Expand current item
+        collapse(index);
+
+        // Prepend all children of current item
+        for (int i=0 ; i < model()->rowCount(index) ; ++i) {
+            worklist.append(index.child(i,0));
+        }
+    }
+}
+
 void KateFileTree::slotDocumentCloseOther()
 {
     QVariant v = model()->data(m_indexContextMenu.parent(), KateFileTreeModel::DocumentTreeRole);
@@ -387,7 +444,7 @@ void KateFileTree::slotRenameFile() {
     }
 
     QUrl newFileUrl = oldFileUrl.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash);
-    newFileUrl.setPath(newFileUrl.path() + QDir::separator() + newFileName);
+    newFileUrl.setPath(newFileUrl.path() + QLatin1Char('/') + newFileName);
 
     if (!newFileUrl.isValid()) {
         return;
